float red;
float green;
float blue;
uint32_t state = 0;

short stepColor = 1;

unsigned short red_out;
unsigned short green_out;
unsigned short blue_out;

unsigned short prev_red_out;
unsigned short prev_green_out;
unsigned short prev_blue_out;

void setup()
{
  pinMode(3, OUTPUT);        // Rosu catod comun
  pinMode(9, OUTPUT);        // Rosu anod comun
  pinMode(5, OUTPUT);        // Verde catod comun
  pinMode(10, OUTPUT);       // Verde anod comun
  pinMode(6, OUTPUT);        // Albastru catod comun
  pinMode(11, OUTPUT);       // Albastru anod comun
  
  pinMode(A5, INPUT);        // Semnal Rosu
  pinMode(A4, INPUT);        // Semnal Verde
  pinMode(A3, INPUT);        // Semnal Albastru

  pinMode(A0, INPUT);        // Jack 3.5mm
  pinMode(8, OUTPUT);        // Modul 0
  pinMode(7, OUTPUT);        // Modul 1
  pinMode(4, OUTPUT);        // Modul 2
  
  pinMode(2, INPUT_PULLUP);  // Buton schimbare mod
  attachInterrupt(digitalPinToInterrupt(2), changeMode, FALLING);
  
  Serial.begin(9600);
}

void changeMode() {
  state = state + 1;
  state = state % 4;
}

// Modifica intensitatea si culoarea RGB-urilor in functie de semnalul primit
void colorShifter(short oldColor, short newColor, float factorChange) {
  if (oldColor-newColor <= 0) {
    newColor = oldColor + min((newColor-oldColor), factorChange);
  } else {
    newColor = newColor - min((oldColor-newColor), factorChange);
  }

  oldColor = newColor;
}

/* 
 * Functia principala folosita in cadrul modului 'Automat'
 * 1-50 : Tranzitie Rosu
 * 51-100: Tranzitie Verde
 * 101-150: Tranzitie Albastru
 * 151-200: Tranzitie Alb
 * 201-260: Ciclare R, G, B, RG, GB, RB
 */
void changePhase() {
  if (stepColor == 1) {
    red_out = green_out = blue_out = 0;
  }
  
  if (stepColor < 51) {
    if (stepColor < 26) {
      red_out = red_out + 10;
    } else {
      red_out = red_out - 10;
    }
  } else if (stepColor < 101) {
    red_out = 0;
    if (stepColor < 76) {
      green_out = green_out + 10;
    } else {
      green_out = green_out - 10;
    }
  } else if (stepColor < 151) {
    green_out = 0;
    if (stepColor < 126) {
      blue_out = blue_out + 10;
    } else {
      blue_out = blue_out - 10;
    }
  } else if (stepColor < 201) {
    if (stepColor == 151) {
      blue_out = 0;
    }
    if (stepColor < 176) {
      red_out = red_out + 10;
      green_out = green_out + 10;
      blue_out = blue_out + 10;
    } else {
      red_out = red_out - 10;
      green_out = green_out - 10;
      blue_out = blue_out - 10;
    }
  } else {
    switch ((stepColor - 201) % 6) {
      case 0:
        red_out = 255;
        green_out = 0;
        blue_out = 0;
        break;
      case 1:
        red_out = 0;
        green_out = 255;
        blue_out = 0;
        break;
      case 2:
        red_out = 0;
        green_out = 0;
        blue_out = 255;
        break;
      case 3:
        red_out = 255;
        green_out = 255;
        blue_out = 0;
        break;
      case 4:
        red_out = 0;
        green_out = 255;
        blue_out = 255;
        break;
      default:
        red_out = 255;
        green_out = 0;
        blue_out = 255;
        break;
    }
  }
  
  stepColor++;
  if (stepColor == 261) {
    stepColor = 1;
  }
}

void loop() {
    // Modul "stins"
    if (state == 0) {
      digitalWrite(4, LOW);
      digitalWrite(7, LOW);
      digitalWrite(8, LOW);
      stepColor = 1;
      
      analogWrite(6, 0);
      analogWrite(3, 0);
      analogWrite(5, 0);
      analogWrite(9, 255);
      analogWrite(11, 255);
      analogWrite(10, 255);
      delay(150);

    // Modul "Manual"
    } else if (state == 1) {
      digitalWrite(4, HIGH);
      digitalWrite(7, LOW);
      digitalWrite(8, LOW);
      stepColor = 1;
      
      red = analogRead(A5);
      green = analogRead(A4);
      blue = analogRead(A3);
      red_out = 255*(red/1023.00);
      green_out = 255*(green/1023.00);
      blue_out = 255*(blue/1023.00);
      
      analogWrite(3, red_out);
      analogWrite(9, 255-red_out);
      analogWrite(5, green_out);
      analogWrite(10, 255-green_out);
      analogWrite(6, blue_out);
      analogWrite(11, 255-blue_out);
      delay(100);

    // Modul Automat
    } else if (state == 2) {
      digitalWrite(4, LOW);
      digitalWrite(7, HIGH);
      digitalWrite(8, LOW);
      
      changePhase();

      analogWrite(3, red_out);
      analogWrite(9, 255-red_out);
      analogWrite(5, green_out);
      analogWrite(10, 255-green_out);
      analogWrite(6, blue_out);
      analogWrite(11, 255-blue_out);

      if (stepColor - 1 > 199) {
        delay(150);
      } else {
        delay(30);
      }

    // Modul input audio (Music mode)
    } else {
      stepColor = 1;
      
      randomSeed(millis());
      digitalWrite(4, LOW);
      digitalWrite(7, LOW);
      digitalWrite(8, HIGH);

      short signalIntensity = analogRead(A0);
      float factorChange = 25.00;
    
      if (signalIntensity <= 20 && signalIntensity >= 6) {
        factorChange = 25.00;
      } else if (signalIntensity <= 40 && signalIntensity >= 21) {
        factorChange = 50.00;
      } else if (signalIntensity <= 60 && signalIntensity >= 41) {
        factorChange = 75.00;
      } else if (signalIntensity > 60) {
        factorChange = 100.00;
      }
      
      if (signalIntensity > 5) {
        red_out = random(255);
        green_out = random(255);
        blue_out = random(255);

        if (signalIntensity <= 60) {
          colorShifter(prev_red_out, red_out, factorChange);
          colorShifter(prev_green_out, green_out, factorChange);
          colorShifter(prev_blue_out, blue_out, factorChange);
        }

        analogWrite(3, red_out);
        analogWrite(9, 255-red_out);
        analogWrite(5, green_out);
        analogWrite(10, 255-green_out);
        analogWrite(6, blue_out);
        analogWrite(11, 255-blue_out);
      }
    }

    delay(10);
}
